import java.util.Enumeration;

import javax.jms.Connection;
import javax.jms.ConnectionFactory;
import javax.jms.DeliveryMode;
import javax.jms.JMSException;
import javax.jms.MapMessage;
import javax.jms.Message;
import javax.jms.MessageConsumer;
import javax.jms.MessageProducer;
import javax.jms.Queue;
import javax.jms.Session;

import org.apache.activemq.ActiveMQConnectionFactory;
import org.apache.activemq.ActiveMQSslConnectionFactory;
import org.apache.activemq.ActiveMQConnection;
import org.apache.activemq.transport.tcp.SslTransport;
import org.apache.activemq.transport.Transport;
import org.apache.activemq.transport.TransportFilter;
import java.security.cert.X509Certificate;

import java.io.File;
import java.io.FileInputStream;
import java.util.Properties;
import java.io.IOException;



public class CheckAMQStatistics {
	private Properties props;
	private String brokerURL;
	private String brokerUserName = "";
	private String brokerUserPasswd = "";
	private String brokerQueueName = "ActiveMQ.Statistics.Broker";
	
	public void setUccConfig(String propsFile) throws IOException {
		props = new Properties();
		FileInputStream fis = new FileInputStream(propsFile);
		props.load(fis);
		fis.close();
	}

	public void setBrokerURL(String url) {
		brokerURL = url;
	}

  	public void setBrokerCredentials(String username, String password) {
		brokerUserName = username;
		brokerUserPasswd = password;
	}

	public void setQueueName(String name) {
		brokerQueueName = name;
	}

	public void run() throws JMSException {
		ConnectionFactory connectionFactory = null;
		if (brokerURL.startsWith("ssl://")) {
			ActiveMQSslConnectionFactory sslFactory = new ActiveMQSslConnectionFactory(brokerURL);
			sslFactory.setKeyStore("file://" + props.getProperty("credential.path"));
			sslFactory.setKeyStorePassword(props.getProperty("credential.password"));
			sslFactory.setTrustStore("file://" + (props.getProperty("truststore.keystorePath") != null ? props.getProperty("truststore.keystorePath") : props.getProperty("credential.path")));
			sslFactory.setTrustStorePassword(props.getProperty("truststore.keystorePassword") != null ? props.getProperty("truststore.keystorePassword") : props.getProperty("credential.password"));
			connectionFactory = sslFactory;
		} else {
			connectionFactory = new ActiveMQConnectionFactory(brokerURL);
		}
 
		Connection connection = null;
		Session session = null;
		MessageConsumer consumer = null;

		if (brokerUserName.equals("")) {
			connection = connectionFactory.createConnection();
		} else {
			connection = connectionFactory.createConnection(brokerUserName, brokerUserPasswd);
		}    		
		connection.start();
		Transport transport = ((ActiveMQConnection)connection).getTransport();
		while (transport instanceof TransportFilter)
			transport = ((TransportFilter)transport).getNext();

		if (transport instanceof SslTransport)  {
			for(X509Certificate cert: ((SslTransport)transport).getPeerCertificates()) {
				System.out.println("certificateExpireDate="+cert.getNotAfter());
				break; // getPeerCertificates() returns an ordered array of peer certificates, with the peer's own certificate first followed by any certificate authorities.
			}
		}
		session = connection.createSession(false, Session.AUTO_ACKNOWLEDGE);
		Queue replyTo = session.createTemporaryQueue();
		consumer = session.createConsumer(replyTo);
   		
		Queue testQueue = session.createQueue(brokerQueueName);
		MessageProducer producer = session.createProducer(testQueue);
		producer.setDeliveryMode(DeliveryMode.NON_PERSISTENT);

		Message msg = session.createMessage();
		msg.setJMSReplyTo(replyTo);
		producer.send(msg);
		producer.close();

		MapMessage reply = (MapMessage) consumer.receive();

		for (Enumeration e = reply.getMapNames();e.hasMoreElements();) {
			String name = e.nextElement().toString();
			System.out.println(name + "=" + reply.getObject(name));
		}
		consumer.close();
		session.close();
		connection.close();			
	}

	public static void main(String[] args) throws IOException, JMSException {
		if (args.length < 2) {
		   System.out.println("USAGE: java CheckAMQStatistics <UCC_CONFIG> <BROKER_URL> [OPTIONAL_BROKER_USERNAME] [OPTIONAL_BROKER_PASSWORD] [OPTIONAL_QUEUE_NAME]");
		   System.exit(1);
		}
		CheckAMQStatistics app = new CheckAMQStatistics();
		app.setUccConfig(args[0]);
		app.setBrokerURL(args[1]);
		if (args.length > 2)
			app.setBrokerCredentials(args[2], args[3]);
		if (args.length == 5)
			app.setQueueName(args[4]);

    try {
      app.run();
      System.exit(0);
    } catch (Exception e) {
      e.printStackTrace();
      System.exit(1);
    }
		
	}
}

