import java.io.BufferedReader;
import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.net.URL;
import java.security.KeyManagementException;
import java.security.KeyStore;
import java.security.KeyStoreException;
import java.security.NoSuchAlgorithmException;
import java.security.NoSuchProviderException;
import java.security.SecureRandom;
import java.security.UnrecoverableKeyException;
import java.security.cert.CertificateException;
import java.util.regex.Matcher;
import java.util.regex.Pattern;
import java.util.Properties;

import javax.net.ssl.HostnameVerifier;
import javax.net.ssl.HttpsURLConnection;
import javax.net.ssl.KeyManagerFactory;
import javax.net.ssl.SSLContext;
import javax.net.ssl.SSLSession;
import javax.net.ssl.SSLSocketFactory;
import javax.net.ssl.TrustManagerFactory;

public class CheckGateway {

  private static Properties load(File propsFile) throws IOException {
    Properties props = new Properties();
    FileInputStream fis = new FileInputStream(propsFile);
    props.load(fis);    
    fis.close();
    return props;
  }
	
	private static SSLSocketFactory getFactory( File pKeyFile, String pKeyType, String pKeyPassword, File pTrustFile, String pTrustPassword ) throws NoSuchAlgorithmException, KeyStoreException, CertificateException, IOException, UnrecoverableKeyException, KeyManagementException, NoSuchProviderException {
		  KeyManagerFactory keyManagerFactory = KeyManagerFactory.getInstance("SunX509");
		  KeyStore keyStore = KeyStore.getInstance(pKeyType==null?"JKS":pKeyType);
      KeyStore trustStore = KeyStore.getInstance("JKS");

		  InputStream keyInput = new FileInputStream(pKeyFile);
		  keyStore.load(keyInput, pKeyPassword.toCharArray());
		  keyInput.close();

		  InputStream key2Input = new FileInputStream(pTrustFile);
		  trustStore.load(key2Input, pTrustPassword.toCharArray());
		  key2Input.close();


		  keyManagerFactory.init(keyStore, pKeyPassword.toCharArray());

		  TrustManagerFactory tmf = TrustManagerFactory.getInstance("PKIX", "SunJSSE");
          tmf.init(trustStore);
		  
		  SSLContext context = SSLContext.getInstance("TLS");
		  context.init(keyManagerFactory.getKeyManagers(), tmf.getTrustManagers(), new SecureRandom());

		  return context.getSocketFactory();
		}

	public static void main(String[] args) {  
		if (args.length != 2) {
			System.out.println("Usage: java CheckGateway GATEWAY_URL UCC_CONFIG_FILE_PATH");
			System.exit(1);
		}
		
		try {
			URL url =  new URL(args[0]);
			HttpsURLConnection con = (HttpsURLConnection) url.openConnection();

      Properties p = load(new File(args[1]));
			con.setSSLSocketFactory(getFactory(new File(p.getProperty("credential.path")), p.getProperty("credential.format"), p.getProperty("credential.password"), new File(p.getProperty("truststore.keystorePath")!=null?p.getProperty("truststore.keystorePath"):p.getProperty("credential.path")),  (p.getProperty("truststore.keystorePath")!=null?p.getProperty("truststore.keystorePassword"):p.getProperty("credential.password"))));
			con.setHostnameVerifier(new HostnameVerifier() {
				public boolean verify(String arg0, SSLSession arg1) {
					return true;
				}
			});
			con.setRequestMethod("GET");
			con.connect();
			printVSites(con.getInputStream());
			
			con.disconnect();
			
		} catch (Exception e) {
      System.out.println(e);
			e.printStackTrace();
      System.exit(1);
		} 

    System.exit(0);

	}


	private static void printVSites(InputStream inputStream) throws IOException {
		BufferedReader br = new BufferedReader(new InputStreamReader(inputStream));
		Pattern p = Pattern.compile("<td> *([^<]*)</td><td><a href='[^']*'>([^<]*)</a></td><td align='right'> *([0-9]*)</td><td><img[^>]*></td><td>([^>]*)</td>");
		String inputLine;

		while ((inputLine = br.readLine()) != null) {
			Matcher m =p.matcher(inputLine);
			while (m.find() == true)
				System.out.println(m.group(1) + " | " + m.group(4) + " | " + m.group(2) + " | " + m.group(3));
		}

		br.close();
	}

}
